//+------------------------------------------------------------------+
//|                                                   RNN_MT4_v1.mq4 |
//|                               Copyright  2012, Yury V. Reshetov |
//|                                          http://reshetov.xnet.uz |
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, Yury V. Reshetov"
#property link      "http://reshetov.xnet.uz"


//----  
extern int          x0 = 50; //   0  100   1
extern int          x1 = 50; //   0  100   1
extern int          x2 = 50; //   0  100   1
extern int          x3 = 50; //   0  100   1
extern int          x4 = 50; //   0  100   1
extern int          x5 = 50; //   0  100   1 
extern int          x6 = 50; //   0  100   1 
extern int          x7 = 50; //   0  100   1
extern double       sl = 500.0; //      
extern double       lots = 1.0;
extern int          mn = 888; //  
static int          prevtime = 0; //    


int init() {
   prevtime = Time[0]; //       
   double sign = tradesSignal();
   Comment("Signal = ", sign);
   return(0);
}

//+------------------------------------------------------------------+
//|                              |
//+------------------------------------------------------------------+
int start() {
   //     
   if (Time[0] == prevtime) {
      return(0);
   }
   prevtime = Time[0];
   
   //    
   if(! IsTradeAllowed()) {
      Print("Trades not allowed");
      again();
      return(0);
   }


   //   
   
   //     
   int total = OrdersTotal(); 
   //   
   for(int i = total - 1; i >= 0; i--) { //   
      if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) continue; 
      //        
      if ((OrderSymbol() == Symbol()) && (OrderMagicNumber() == mn)) {
         //   , 
         double signal = tradesSignal();
         Comment("Signal = ", signal);
         return(0); 
      }
   }

   //   
   double sp = tradesSignal();

   double stoploss = 0.0;
   double takeprofit = 0.0;
   
   double lt = lots;
   
   if (IsOptimization()) {
      lt = getLots(MathAbs(sp));
   }
   
   
   //    
   RefreshRates();
   
   string s = "";
   int ticket = -1;
   
   if (sp < 0.0) {
      stoploss = Bid - sl * Point;
      takeprofit = Bid + sl * Point;
      ticket = OrderSend(Symbol(), OP_BUY, lt, Ask, 1, stoploss, takeprofit, WindowExpertName(), mn, 0, Blue); 
      if (ticket < 0) {
         Print("OrderSend failed with error #", GetLastError());
         again();      
      } else {
         s = "Open " + Symbol() +  " #" + DoubleToStr(ticket, 0) + " Buy , Price:" + Ask + ", " + DoubleToStr(lots, 2) + " lots, SL: " + DoubleToStr(stoploss, 5) + ", TP: " + DoubleToStr(takeprofit, 5);
         SendMail("Open Buy position of " + Symbol(), s);
      }
   } else {
      stoploss = Ask + sl * Point;
      takeprofit = Ask - sl * Point;
      ticket = OrderSend(Symbol(), OP_SELL, lt, Bid, 1, stoploss, takeprofit, WindowExpertName(), mn, 0, Red); 
      if (ticket < 0) {
         Print("OrderSend failed with error #", GetLastError());
         again();
      } else {
         s = "Open " + Symbol() +  " #" + DoubleToStr(ticket, 0) + " Sell , Price:" + Bid + ", " + DoubleToStr(lots, 2) + " lots, SL: " + DoubleToStr(stoploss, 5) + ", TP: " + DoubleToStr(takeprofit, 5);
         SendMail("Open Sell position of " + Symbol(), s);
      }
   }
//-- Exit --
   return(0);
}


//     

double tradesSignal() {
   //   
   double a1 = finput(9, 0);
   double a2 = sigmoid(a1 - finput(9, 1));
   double a3 = sigmoid(a1 - finput(9, 2));

   //       
   double result = getProbability(a1, a2, a3);
   
   string s = Symbol() + ", Probability for Short (Sell) position: " + DoubleToStr(result, 4);

   if (result > 0.5) {
      Print(s);
   } else {
      double r = 1.0 - result;
      s = Symbol() + ", Probability for Long (Buy) position: " + DoubleToStr(r, 4);
      Print(s);
   }
   
   SendMail(s, s);
   //  ,    0  1    -1  +1
   result = result * 2.0 - 1.0;
   Print("Result = ", result);
   return (result);
}


//   . 
//       
// p1, p2, p3 -         0  1
double getProbability(double p1, double p2, double p3) {

   double y0 = x0;
   double y1 = x1;
   double y2 = x2;
   double y3 = x3;
   double y4 = x4;
   double y5 = x5;
   double y6 = x6;
   double y7 = x7;

   double pn1 = 1.0 - p1;
   double pn2 = 1.0 - p2;
   double pn3 = 1.0 - p3;

   //    
   double probability = 
   pn1 * (pn2 * (pn3 * y0 +
   p3 * y1) +
   p2 * (pn3 * y2 +
   p3 * y3)) +
   p1 * (pn2 * (pn3 * y4 +
   p3 * y5) +
   p2 * (pn3 * y6 +
   p3 * y7)); 

   //    
   probability = probability / 100.0;


   return (probability);

}

//    start()     30  
void again() {
   prevtime = Time[1];
   Sleep(30000);
}


//  ,      0  1
// p -  
// shift -      
double finput(int p, int shift) { 
  double result = iRSI(Symbol(), 0, 18, PRICE_OPEN, shift * p) / 100.0;
  return (result);
}

//  
double sigmoid(double x) {
   return((x + 1.0) / 2.0);
}

//         
double getLots(double sp) {
   double result = lots * sp;
   if (result < 0.1) {
      result = 0.1;
   }
   result = NormalizeDouble(result, 1);
   return(result);
}


